#include <MySensor.h>  
#include <SPI.h>
#include <DallasTemperature.h>
#include <OneWire.h>

unsigned long SLEEP_TIME = 300000; // Sleep time between reports (in milliseconds)

#define ONE_WIRE_POWER 5 // Pin where dallas sensor is powered 
#define ONE_WIRE_BUS 2 // Pin where dallas sensor is connected 

OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature sensors(&oneWire);
boolean receivedConfig = false;
boolean metric = true; 

#define DIGITAL_INPUT_SENSOR 3   // The digital input you attached your motion sensor.  (Only 2 and 3 generates interrupt!)
#define INTERRUPT DIGITAL_INPUT_SENSOR-2 // Usually the interrupt = pin -2 (on uno/nano anyway)
#define CHILD_ID_TEMP 1   // Id of Temp
#define CHILD_ID_MOTION 2   // Id of Motion
#define CHILD_ID_VOLTAGE 3   // Id of Voltage

int BATTERY_SENSE_PIN = A0;  // select the input pin for the battery sense point
int oldValue=-1;

boolean pinTriggered=0;//waitTime is number of seconds to hol in while loop
long lastDebounceTime = 0;  // the last time the output pin was toggled
long debounceDelay = 50;    // the debounce time; increase if the output flickers

MySensor gw;
// Initialize motion message
MyMessage msgTemp(CHILD_ID_TEMP, V_TEMP);
MyMessage msgMot(CHILD_ID_MOTION, V_TRIPPED);
MyMessage msgVolt(CHILD_ID_VOLTAGE, V_VOLTAGE);

void setup()  
{  
  Serial.begin(115200); // managed on the Mysensor config file Myconfig.h with DEBUG Option
  
  // Startup OneWire 
    pinMode(ONE_WIRE_POWER,OUTPUT);
  
  // use the 1.1 V internal reference for Battery Voltage
  analogReference(INTERNAL);

  gw.begin();

  // Send the sketch version information to the gateway and Controller
  gw.sendSketchInfo("Motion Temp w/ Batt", "1.0");

  pinMode(DIGITAL_INPUT_SENSOR, INPUT);      // sets the motion sensor digital pin as input
    digitalWrite(DIGITAL_INPUT_SENSOR,HIGH);   // Activate internal pull-up
    
  // Register all sensors to gw (they will be created as child devices)
  gw.present(CHILD_ID_TEMP, S_TEMP);
  gw.present(CHILD_ID_MOTION, S_MOTION);
  gw.present(CHILD_ID_VOLTAGE, S_POWER);  

  check_batt();  
}

void loop()
{
  if (pinTriggered)
  {
    Serial.println(millis()-lastDebounceTime);
    // Read digital motion value
    boolean value = digitalRead(DIGITAL_INPUT_SENSOR);
    // If the switch changed, due to noise or pressing
    if (value != oldValue) {
      {
        lastDebounceTime = millis();
        // Send in the new value
        gw.send(msgMot.set(value==HIGH ? 1 : 0));

        oldValue = value;
        Serial.print("Mot: ");
        Serial.println(value);
      }
    }

    pinTriggered=0;
    // }
  }
  else
  {

    check_batt();

  // Turn onn and Fetch temperatures from Dallas sensors
    digitalWrite(ONE_WIRE_POWER,HIGH);
    delay(50);
      sensors.begin();
      sensors.requestTemperatures(); 

  // Read temperatures and send them to controller 
  float temperature = static_cast<float>(static_cast<int>((gw.getConfig().isMetric?sensors.getTempCByIndex(0):sensors.getTempFByIndex(0)) * 10.)) / 10.;

  //Serial.print("Temperature: ");
  //Serial.print(temperature);
  //Serial.println(" C");
  
      // Send in the new temperature
      gw.send(msgTemp.set(temperature,1));  
  
  //turn off temp sensor
  digitalWrite(ONE_WIRE_POWER,LOW);
  }
  


  // Sleep until interrupt comes in on motion sensor. Send update every 5 minutes.
  pinTriggered = gw.sleep(INTERRUPT, CHANGE, SLEEP_TIME);
}

void check_batt()
{
  // get the battery Voltage
  int sensorValue = analogRead(BATTERY_SENSE_PIN);
  Serial.println(sensorValue);

  // 1M, 470K divider across battery and using internal ADC ref of 1.1V
  // Sense point is bypassed with 0.1 uF cap to reduce noise at that point
  // ((1e6+470e3)/470e3)*1.1 = Vmax = 3.44 Volts
  // 3.44/1023 = Volts per bit = 0.003363075
  float batteryV  = sensorValue * 0.003363075;
  int batteryPcnt = sensorValue / 10;

  //Serial.print("Battery Voltage: ");
  //Serial.print(batteryV);
  //Serial.println(" V");

  //Serial.print("Battery percent: ");
  //Serial.print(batteryPcnt);
  //Serial.println(" %");

//    gw.sendBatteryLevel(batteryPcnt);
      //send using gw for PiDome
      gw.send(msgVolt.set(batteryPcnt, 1));    
   
}
